import React, { useEffect, useContext, useState } from 'react'
import { AppContext } from 'components/Contexts/AppContextProvider'
import {
  TimeStep,
  IPerformanceIndicator,
  ITimePeriod,
} from 'services/dataConsumptionContracts'
import { withClient, Client } from 'cozy-client'
import PerformanceIndicator from 'components/ContentComponents/PerformanceIndicator/PerformanceIndicator'
import IndicatorContainerSwitcher from 'components/ContainerComponents/IndicatorsContainer/IndicatorContainerSwitcher'
import ConsumptionPeriodSelector from 'services/consumptionPeriodSelectorService'
import ConsumptionDataManager from 'services/consumptionDataManagerService'
import PerformanceIndicatorAggregateCalculator from 'services/performanceIndicatorAggregateCalculatorService'
import { translate } from 'cozy-ui/react/I18n'
import { convertDateByTimeStep } from 'utils/utils'

interface MultiFluidIndicatorsContainerProps {
  timeStep: TimeStep
  setIndicatorsLoaded(): void
  client: Client
  t: Function
}

const MultiFluidIndicatorsContainer: React.FC<MultiFluidIndicatorsContainerProps> = ({
  timeStep,
  setIndicatorsLoaded,
  client,
  t,
}: MultiFluidIndicatorsContainerProps) => {
  const { fluidTypes } = useContext(AppContext)
  const [performanceIndicators, setPerformanceIndicators] = useState<
    IPerformanceIndicator[]
  >([])

  const [
    currentTimePeriod,
    setCurrentTimePeriod,
  ] = useState<ITimePeriod | null>(null)
  const [
    comparisonTimePeriod,
    setComparisonTimePeriod,
  ] = useState<ITimePeriod | null>(null)
  const [isLoaded, setIsLoaded] = useState<boolean>(false)

  useEffect(() => {
    let subscribed = true
    async function populatePerformanceIndicators() {
      const cdm = new ConsumptionDataManager(client)
      const lastDate = await cdm.fetchLastDateData(fluidTypes)

      if (subscribed && lastDate) {
        const consumptionPeriodSelector = new ConsumptionPeriodSelector()
        const periods = consumptionPeriodSelector.getTimePeriods(
          lastDate,
          fluidTypes,
          timeStep
        )

        const fetchedPerformanceIndicators = await cdm.getPerformanceIndicators(
          periods.timePeriod,
          timeStep,
          fluidTypes,
          periods.comparisonTimePeriod
        )

        if (fetchedPerformanceIndicators) {
          setPerformanceIndicators(fetchedPerformanceIndicators)
          setCurrentTimePeriod(periods.timePeriod)
          setComparisonTimePeriod(periods.comparisonTimePeriod)
        }
      }
      setIndicatorsLoaded()
      setIsLoaded(true)
    }
    populatePerformanceIndicators()
    return () => {
      subscribed = false
    }
  }, [timeStep, fluidTypes])

  return (
    <>
      {isLoaded ? (
        <div className="fi-root">
          <div className="fi-content">
            <div className="fi-header text-14-normal-uppercase">
              {t('COMMON.CONSO_CARDS_LABEL')}{' '}
              {convertDateByTimeStep(currentTimePeriod, timeStep, true)}
            </div>
            <PerformanceIndicator
              performanceIndicator={PerformanceIndicatorAggregateCalculator.aggregatePerformanceIndicators(
                performanceIndicators
              )}
              timePeriodText={convertDateByTimeStep(
                comparisonTimePeriod,
                timeStep
              )}
              timeStep={timeStep}
            />
            <IndicatorContainerSwitcher
              performanceIndicators={performanceIndicators}
              timeStep={timeStep}
              comparisonTimePeriod={comparisonTimePeriod}
            />
          </div>
        </div>
      ) : null}
    </>
  )
}

export default translate()(withClient(MultiFluidIndicatorsContainer))
